#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.botswana import Botswana
from tests.common import CommonCountryTests


class TestBotswana(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Botswana, years_non_observed=range(2010, 2024))

    def test_special_holidays(self):
        self.assertHoliday("2019-07-02")

    def test_new_years_day(self):
        name = "New Year's Day"
        name_holiday = f"{name} Holiday"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        self.assertHolidayName(name_holiday, (f"{year}-01-02" for year in self.full_range))

        obs_dts = (
            "2012-01-03",
            "2017-01-03",
            "2023-01-03",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        dts_holiday = (
            "2005-01-03",
            "2011-01-03",
            "2022-01-03",
        )
        self.assertHolidayName(f"{name_holiday} (observed)", dts_holiday)
        self.assertNoNonObservedHoliday(obs_dts, dts_holiday)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_holy_saturday(self):
        name = "Holy Saturday"
        self.assertHolidayName(
            name,
            "2020-04-11",
            "2021-04-03",
            "2022-04-16",
            "2023-04-08",
            "2024-03-30",
            "2025-04-19",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHoliday(f"{year}-05-01" for year in self.full_range)

        obs_dts = (
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        dts_sat = (
            "2021-05-03",
            "2027-05-03",
            "2032-05-03",
        )
        self.assertHolidayName(f"{name} Holiday", dts_sat)
        self.assertNoNonObservedHoliday(obs_dts, dts_sat)

    def test_presidents_day(self):
        name = "President's Day"
        name_holiday = f"{name} Holiday"
        self.assertHolidayName(
            name,
            "2020-07-20",
            "2021-07-19",
            "2022-07-18",
            "2023-07-17",
            "2024-07-15",
            "2025-07-21",
        )
        self.assertHolidayName(name, self.full_range)
        self.assertHolidayName(
            name_holiday,
            "2020-07-21",
            "2021-07-20",
            "2022-07-19",
            "2023-07-18",
            "2024-07-16",
            "2025-07-22",
        )
        self.assertHolidayName(name_holiday, self.full_range)

    def test_botswana_day(self):
        name = "Botswana Day"
        name_holiday = f"{name} Holiday"
        self.assertHolidayName(name, (f"{year}-09-30" for year in self.full_range))
        self.assertHolidayName(name_holiday, (f"{year}-10-01" for year in self.full_range))

        obs_dts = (
            "2007-10-02",
            "2012-10-02",
            "2018-10-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        dts_holiday = (
            "2006-10-02",
            "2017-10-02",
            "2023-10-02",
        )
        self.assertHolidayName(f"{name_holiday} (observed)", dts_holiday)
        self.assertNoNonObservedHoliday(obs_dts, dts_holiday)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))

        obs_dts = (
            "2011-12-27",
            "2016-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))

        obs_dts = (
            "2004-12-27",
            "2010-12-27",
            "2021-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        dts_holiday = (
            "2020-12-28",
            "2026-12-28",
            "2037-12-28",
        )
        self.assertHolidayName(f"{name} Holiday", dts_holiday)
        self.assertNoNonObservedHoliday(obs_dts, dts_holiday)

    def test_2021(self):
        self.assertHolidayDatesInYear(
            2021,
            "2021-01-01",
            "2021-01-02",
            "2021-04-02",
            "2021-04-03",
            "2021-04-05",
            "2021-05-01",
            "2021-05-03",
            "2021-05-13",
            "2021-07-01",
            "2021-07-19",
            "2021-07-20",
            "2021-09-30",
            "2021-10-01",
            "2021-12-25",
            "2021-12-26",
            "2021-12-27",
        )

    def test_2022(self):
        self.assertHolidayDatesInYear(
            2022,
            "2022-01-01",
            "2022-01-02",
            "2022-01-03",
            "2022-04-15",
            "2022-04-16",
            "2022-04-18",
            "2022-05-01",
            "2022-05-02",
            "2022-05-26",
            "2022-07-01",
            "2022-07-18",
            "2022-07-19",
            "2022-09-30",
            "2022-10-01",
            "2022-12-25",
            "2022-12-26",
            "2022-12-27",
        )

    def test_2023(self):
        self.assertHolidayDatesInYear(
            2023,
            "2023-01-01",
            "2023-01-02",
            "2023-01-03",
            "2023-04-07",
            "2023-04-08",
            "2023-04-10",
            "2023-05-01",
            "2023-05-18",
            "2023-07-01",
            "2023-07-17",
            "2023-07-18",
            "2023-09-30",
            "2023-10-01",
            "2023-10-02",
            "2023-12-25",
            "2023-12-26",
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.brazil import Brazil
from tests.common import CommonCountryTests


class TestBrazil(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(
            Brazil,
            years_all_subdivs=range(1995, 2050),
            years_subdiv_são_paulo_capital=range(1967, 2050),
        )

    def test_new_years_day(self):
        self.assertHolidayName(
            "Confraternização Universal", (f"{year}-01-01" for year in self.full_range)
        )

    def test_republic_constitution_day(self):
        name = "Constituição da República"
        self.assertHolidayName(name, (f"{year}-02-24" for year in range(1892, 1931)))
        self.assertNoHolidayName(name, range(self.start_year, 1892), range(1931, self.end_year))

    def test_discovery_of_brazil(self):
        name = "Descobrimento do Brasil"
        self.assertHolidayName(
            name, (f"{year}-05-03" for year in (*range(self.start_year, 1931), *range(1936, 1949)))
        )
        self.assertNoHolidayName(name, range(1931, 1936), range(1949, self.end_year))

    def test_abolition_of_slavery_in_brazil(self):
        name = "Abolição da escravidão no Brasil"
        self.assertHolidayName(name, (f"{year}-05-13" for year in range(self.start_year, 1931)))
        self.assertNoHolidayName(name, range(1931, self.end_year))

    def test_freedom_and_independence_of_american_peoples(self):
        name = "Liberdade e Independência dos Povos Americanos"
        self.assertHolidayName(name, (f"{year}-07-14" for year in range(self.start_year, 1931)))
        self.assertNoHolidayName(name, range(1931, self.end_year))

    def test_good_friday(self):
        name = "Sexta-feira Santa"
        self.assertHolidayName(
            name,
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_tiradentes_day(self):
        name = "Tiradentes"
        years_absent = {1931, 1932}
        self.assertHolidayName(
            name, (f"{year}-04-21" for year in self.full_range if year not in years_absent)
        )
        self.assertNoHolidayName(name, years_absent)

    def test_workers_day(self):
        name = "Dia do Trabalhador"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1925, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1925))

    def test_independence_day(self):
        self.assertHolidayName(
            "Independência do Brasil", (f"{year}-09-07" for year in self.full_range)
        )

    def test_discovery_of_america(self):
        name = "Descobrimento da América"
        self.assertHolidayName(
            name, (f"{year}-10-12" for year in (*range(self.start_year, 1931), *range(1936, 1949)))
        )
        self.assertNoHolidayName(name, range(1931, 1936), range(1949, self.end_year))

    def test_our_lady_of_aparecida(self):
        name = "Nossa Senhora Aparecida"
        self.assertHolidayName(name, (f"{year}-10-12" for year in range(1980, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1980))

    def test_all_souls_day(self):
        self.assertHolidayName("Finados", (f"{year}-11-02" for year in self.full_range))

    def test_republic_proclamation_day(self):
        self.assertHolidayName(
            "Proclamação da República", (f"{year}-11-15" for year in self.full_range)
        )

    def test_black_awareness_day(self):
        name_black = "Consciência Negra"
        name_zumbi = "Dia Nacional de Zumbi e da Consciência Negra"
        self.assertHolidayName(
            name_zumbi, (f"{year}-11-20" for year in range(2024, self.end_year))
        )
        self.assertNoHolidayName(name_zumbi, range(self.start_year, 2024))

        subdiv_start_years = {
            "AL": 1996,
            "AM": 2010,
            "AP": 2008,
            "MT": 2003,
            "RJ": 2002,
            "São Paulo Capital": 2004,
        }
        for subdiv, holidays in self.subdiv_holidays.items():
            if start_year := subdiv_start_years.get(subdiv):
                self.assertHolidayName(
                    name_black, holidays, (f"{year}-11-20" for year in range(start_year, 2024))
                )
                self.assertNoHolidayName(
                    name_black, holidays, range(1996, start_year), range(2024, self.end_year)
                )
            else:
                self.assertNoHolidayName(name_black, holidays)

    def test_christmas_day(self):
        name = "Natal"
        self.assertHolidayName(name, (f"{year}-12-25" for year in range(1922, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1922))

    def test_carnaval(self):
        name = "Carnaval"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-02-24",
            "2020-02-25",
            "2021-02-15",
            "2021-02-16",
            "2022-02-28",
            "2022-03-01",
            "2023-02-20",
            "2023-02-21",
            "2024-02-12",
            "2024-02-13",
            "2025-03-03",
            "2025-03-04",
        )
        self.assertOptionalHolidayNameCount(name, 2, self.full_range)

        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RJ":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-02-25",
                    "2021-02-16",
                    "2022-03-01",
                    "2023-02-21",
                    "2024-02-13",
                    "2025-03-04",
                )
                self.assertHolidayName(name, holidays, range(2009, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1996, 2009))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_ash_wednesday(self):
        name = "Início da Quaresma"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name,
            "2020-02-26",
            "2021-02-17",
            "2022-03-02",
            "2023-02-22",
            "2024-02-14",
            "2025-03-05",
        )
        self.assertOptionalHolidayName(name, self.full_range)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        self.assertNoHolidayName(name)
        dts = (
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
            "2025-06-19",
        )
        self.assertOptionalHolidayName(name, dts)
        self.assertOptionalHolidayName(name, self.full_range)

        # PUBLIC.
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "São Paulo Capital":
                self.assertHolidayName(name, holidays, dts)
                self.assertHolidayName(name, holidays, range(1996, self.end_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_public_servants_day(self):
        name = "Dia do Servidor Público"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-10-28" for year in self.full_range))

    def test_christmas_eve(self):
        name = "Véspera de Natal"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Véspera de Ano-Novo"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_evangelical_day(self):
        name = "Dia do Evangélico"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "AC":
                    self.assertHolidayName(
                        name,
                        holidays,
                        (f"{year}-01-23" for year in range(2005, 2009)),
                        "2013-01-25",
                        "2014-01-24",
                        "2018-01-26",
                        "2019-01-25",
                        "2020-01-24",
                    )
                    self.assertHolidayName(name, holidays, range(2009, self.end_year))
                    self.assertNoHolidayName(name, holidays, range(1996, 2005))
                case "AL":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-11-30" for year in range(2013, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(1996, 2013))
                case "DF":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-11-30" for year in range(1996, self.end_year))
                    )
                case "RO":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-06-18" for year in range(2002, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(1996, 2002))
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_international_womens_day(self):
        name = "Dia Internacional da Mulher"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AC":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-03-08" for year in range(2002, 2009)),
                    "2011-03-11",
                    "2012-03-09",
                    "2016-03-11",
                    "2017-03-10",
                    "2018-03-09",
                    "2022-03-11",
                    "2023-03-10",
                )
                self.assertHolidayName(name, holidays, range(2009, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1996, 2002))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_founding_of_acre(self):
        name = "Aniversário do Acre"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AC":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-15" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_amazonia_day(self):
        name = "Dia da Amazônia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AC":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-09-05" for year in range(2004, 2009)),
                    "2012-09-07",
                    "2013-09-06",
                    "2017-09-08",
                    "2018-09-07",
                    "2019-09-06",
                )
                self.assertHolidayName(name, holidays, range(2009, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1996, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_signing_of_the_petropolis_treaty(self):
        name = "Assinatura do Tratado de Petrópolis"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AC":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-11-17" for year in range(1996, 2009)),
                    "2009-11-20",
                    "2010-11-19",
                    "2011-11-18",
                    "2015-11-20",
                    "2016-11-18",
                    "2020-11-20",
                    "2021-11-19",
                    "2022-11-18",
                )
                self.assertHolidayName(name, holidays, range(2009, self.end_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_johns_day(self):
        name = "São João"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-24" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_peters_day(self):
        name = "São Pedro"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-29" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_political_emancipation_of_algoas(self):
        name = "Emancipação Política de Alagoas"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AL":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-16" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_elevation_of_amazonas_to_province(self):
        name = "Elevação do Amazonas à categoria de província"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AM":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-05" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_josephs_day(self):
        name = "São José"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "AP":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-03-19" for year in range(2003, self.end_year))
                    )
                    self.assertNoHolidayName(name, holidays, range(1996, 2003))
                case "CE":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-03-19" for year in range(1996, self.end_year))
                    )
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_saint_james_day(self):
        name = "São Tiago"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-25" for year in range(2012, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2012))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_creation_of_the_federal_territory(self):
        name = "Criação do Território Federal"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "AP":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-13" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_bahia_independence_day(self):
        name = "Independência da Bahia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "BA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-02" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_abolition_of_slavery_in_ceara(self):
        name = "Abolição da escravidão no Ceará"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "CE":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-25" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_our_lady_of_assumption(self):
        name = "Nossa Senhora da Assunção"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "CE":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-15" for year in range(2004, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_foundation_of_brasilia(self):
        name = "Fundação de Brasília"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "DF":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-21" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_our_lady_of_penha(self):
        name = "Nossa Senhora da Penha"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "ES":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-04-20",
                    "2021-04-12",
                    "2022-04-25",
                    "2023-04-17",
                    "2024-04-08",
                )
                self.assertHolidayName(name, holidays, range(2020, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1996, 2020))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_foundation_of_goias_city(self):
        name = "Fundação da cidade de Goiás"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "GO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-26" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_foundation_of_goiania(self):
        name = "Pedra fundamental de Goiânia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "GO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-24" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_maranhao_joining_to_independence_of_brazil(self):
        name = "Adesão do Maranhão à independência do Brasil"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "MA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-28" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_tiradentes_execution(self):
        name = "Execução de Tiradentes"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "MG":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-21" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_state_creation_day(self):
        name = "Criação do Estado"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            match subdiv:
                case "MS":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-10-11" for year in range(1996, self.end_year))
                    )
                case "RO":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-01-04" for year in range(1996, self.end_year))
                    )
                case "RR" | "TO":
                    self.assertHolidayName(
                        name, holidays, (f"{year}-10-05" for year in range(1996, self.end_year))
                    )
                case _:
                    self.assertNoHolidayName(name, holidays)

    def test_grao_para_joining_to_independence_of_brazil(self):
        name = "Adesão do Grão-Pará à independência do Brasil"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PA":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-15" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_state_founding_day(self):
        name = "Fundação do Estado"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PB":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-05" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_pernambuco_revolution(self):
        name = "Revolução Pernambucana de 1817"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PE":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2008-03-02",
                    "2009-03-01",
                    "2010-03-07",
                    "2011-03-06",
                    "2012-03-04",
                    "2013-03-03",
                    "2014-03-02",
                    "2015-03-01",
                    "2016-03-06",
                    "2017-03-05",
                )
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-06" for year in range(2018, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2008))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_piaui_day(self):
        name = "Dia do Piauí"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PI":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-19" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_our_lady_of_rocio(self):
        name = "Nossa Senhora do Rocio"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PR":
                self.assertHolidayName(
                    name, holidays, (f"{year}-11-15" for year in range(2014, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2014))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_political_emancipation_of_parana(self):
        name = "Emancipação do Paraná"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "PR":
                self.assertHolidayName(
                    name, holidays, (f"{year}-12-19" for year in range(1996, 2014))
                )
                self.assertNoHolidayName(name, holidays, range(2014, self.end_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_georges_day(self):
        name = "São Jorge"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RJ":
                self.assertHolidayName(
                    name, holidays, (f"{year}-04-23" for year in range(2008, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2008))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_rio_grande_de_norte_day(self):
        name = "Dia do Rio Grande do Norte"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-07" for year in range(2000, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2000))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_uruacu_and_cunhua_martyrs_day(self):
        name = "Mártires de Cunhaú e Uruaçu"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RN":
                self.assertHolidayName(
                    name, holidays, (f"{year}-10-03" for year in range(2007, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 2007))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_gaucho_day(self):
        name = "Dia do Gaúcho"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "RS":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-20" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_santa_catarina_state_day(self):
        name = "Dia do Estado de Santa Catarina"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2004-08-11",
                    "2005-08-14",
                    "2006-08-13",
                    "2007-08-12",
                    "2018-08-12",
                    "2019-08-11",
                    "2020-08-16",
                    "2021-08-15",
                    "2022-08-14",
                    "2023-08-13",
                    "2024-08-11",
                )
                self.assertHolidayName(name, holidays, range(2004, self.end_year))
                self.assertNoHolidayName(name, holidays, range(1996, 2004))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_catherine_of_alexandria_day(self):
        name = "Dia de Santa Catarina de Alexandria"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SC":
                self.assertHolidayName(
                    name,
                    holidays,
                    "1996-11-25",
                    "1997-11-25",
                    "1998-11-25",
                    "1999-11-28",
                    "2000-11-26",
                    "2004-11-25",
                    "2005-11-27",
                    "2018-11-25",
                    "2019-12-01",
                    "2020-11-29",
                    "2021-11-28",
                    "2022-11-27",
                    "2023-11-26",
                    "2024-12-01",
                )
                self.assertHolidayName(name, holidays, range(1996, self.end_year))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_political_emancipation_of_sergipe(self):
        name = "Emancipação política de Sergipe"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "SE":
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-08" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_constitutionalist_revolution(self):
        name = "Revolução Constitucionalista"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv in {"SP", "São Paulo Capital"}:
                self.assertHolidayName(
                    name, holidays, (f"{year}-07-09" for year in range(1997, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, 1996)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sao_paulo_city_anniversary(self):
        name = "Aniversário da Cidade de São Paulo"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "São Paulo Capital":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-25" for year in range(1968, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1968))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_autonomy_day(self):
        name = "Dia da Autonomia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-03-18" for year in range(1998, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(1996, 1998))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_our_lady_of_nativity(self):
        name = "Nossa Senhora da Natividade"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "TO":
                self.assertHolidayName(
                    name, holidays, (f"{year}-09-08" for year in range(1996, self.end_year))
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Confraternização Universal"),
            ("2023-01-04", "Criação do Estado"),
            ("2023-01-23", "Dia do Evangélico"),
            ("2023-01-25", "Aniversário da Cidade de São Paulo"),
            ("2023-02-20", "Carnaval"),
            ("2023-02-21", "Carnaval"),
            ("2023-02-22", "Início da Quaresma"),
            ("2023-03-06", "Revolução Pernambucana de 1817"),
            ("2023-03-10", "Dia Internacional da Mulher"),
            ("2023-03-18", "Dia da Autonomia"),
            ("2023-03-19", "São José"),
            ("2023-03-25", "Abolição da escravidão no Ceará"),
            ("2023-04-07", "Sexta-feira Santa"),
            ("2023-04-17", "Nossa Senhora da Penha"),
            ("2023-04-21", "Execução de Tiradentes; Fundação de Brasília; Tiradentes"),
            ("2023-04-23", "São Jorge"),
            ("2023-05-01", "Dia do Trabalhador"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-15", "Aniversário do Acre"),
            ("2023-06-18", "Dia do Evangélico"),
            ("2023-06-24", "São João"),
            ("2023-06-29", "São Pedro"),
            ("2023-07-02", "Independência da Bahia"),
            ("2023-07-08", "Emancipação política de Sergipe"),
            ("2023-07-09", "Revolução Constitucionalista"),
            ("2023-07-25", "São Tiago"),
            ("2023-07-26", "Fundação da cidade de Goiás"),
            ("2023-07-28", "Adesão do Maranhão à independência do Brasil"),
            ("2023-08-05", "Fundação do Estado"),
            ("2023-08-07", "Dia do Rio Grande do Norte"),
            ("2023-08-13", "Dia do Estado de Santa Catarina"),
            (
                "2023-08-15",
                "Adesão do Grão-Pará à independência do Brasil; Nossa Senhora da Assunção",
            ),
            ("2023-09-05", "Elevação do Amazonas à categoria de província"),
            ("2023-09-07", "Independência do Brasil"),
            ("2023-09-08", "Dia da Amazônia; Nossa Senhora da Natividade"),
            ("2023-09-13", "Criação do Território Federal"),
            ("2023-09-16", "Emancipação Política de Alagoas"),
            ("2023-09-20", "Dia do Gaúcho"),
            ("2023-10-03", "Mártires de Cunhaú e Uruaçu"),
            ("2023-10-05", "Criação do Estado"),
            ("2023-10-11", "Criação do Estado"),
            ("2023-10-12", "Nossa Senhora Aparecida"),
            ("2023-10-19", "Dia do Piauí"),
            ("2023-10-24", "Pedra fundamental de Goiânia"),
            ("2023-10-28", "Dia do Servidor Público"),
            ("2023-11-02", "Finados"),
            ("2023-11-15", "Nossa Senhora do Rocio; Proclamação da República"),
            ("2023-11-17", "Assinatura do Tratado de Petrópolis"),
            ("2023-11-20", "Consciência Negra"),
            ("2023-11-26", "Dia de Santa Catarina de Alexandria"),
            ("2023-11-30", "Dia do Evangélico"),
            ("2023-12-24", "Véspera de Natal"),
            ("2023-12-25", "Natal"),
            ("2023-12-31", "Véspera de Ano-Novo"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "Universal Fraternization Day"),
            ("2023-01-04", "State Creation Day"),
            ("2023-01-23", "Evangelical Day"),
            ("2023-01-25", "São Paulo City Anniversary"),
            ("2023-02-20", "Carnival"),
            ("2023-02-21", "Carnival"),
            ("2023-02-22", "Ash Wednesday"),
            ("2023-03-06", "Pernambuco Revolution of 1817"),
            ("2023-03-10", "International Women's Day"),
            ("2023-03-18", "Autonomy Day"),
            ("2023-03-19", "Saint Joseph's Day"),
            ("2023-03-25", "Abolition of slavery in Ceará"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-17", "Our Lady of Penha"),
            ("2023-04-21", "Founding of Brasilia; Tiradentes' Day; Tiradentes' Execution"),
            ("2023-04-23", "Saint George's Day"),
            ("2023-05-01", "Worker's Day"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-06-15", "Founding of Acre"),
            ("2023-06-18", "Evangelical Day"),
            ("2023-06-24", "Saint John's Day"),
            ("2023-06-29", "Saint Peter's Day"),
            ("2023-07-02", "Bahia Independence Day"),
            ("2023-07-08", "Sergipe Political Emancipation Day"),
            ("2023-07-09", "Constitutionalist Revolution"),
            ("2023-07-25", "Saint James' Day"),
            ("2023-07-26", "Foundation of Goiás city"),
            ("2023-07-28", "Maranhão joining to independence of Brazil"),
            ("2023-08-05", "State Founding Day"),
            ("2023-08-07", "Rio Grande do Norte Day"),
            ("2023-08-13", "Santa Catarina State Day"),
            ("2023-08-15", "Grão-Pará joining to independence of Brazil; Our Lady of Assumption"),
            ("2023-09-05", "Elevation of Amazonas to province"),
            ("2023-09-07", "Independence Day"),
            ("2023-09-08", "Amazonia Day; Our Lady of Nativity"),
            ("2023-09-13", "Creation of the Federal Territory"),
            ("2023-09-16", "Political Emancipation of Alagoas"),
            ("2023-09-20", "Gaucho Day"),
            ("2023-10-03", "Uruaçu and Cunhaú Martyrs Day"),
            ("2023-10-05", "State Creation Day"),
            ("2023-10-11", "State Creation Day"),
            ("2023-10-12", "Our Lady of Aparecida"),
            ("2023-10-19", "Piauí Day"),
            ("2023-10-24", "Foundation of Goiânia"),
            ("2023-10-28", "Public Servant's Day"),
            ("2023-11-02", "All Souls' Day"),
            ("2023-11-15", "Our Lady of Rocio; Republic Proclamation Day"),
            ("2023-11-17", "Signing of the Petropolis Treaty"),
            ("2023-11-20", "Black Awareness Day"),
            ("2023-11-26", "Saint Catherine of Alexandria Day"),
            ("2023-11-30", "Evangelical Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-31", "New Year's Eve"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "День всесвітнього братання"),
            ("2023-01-04", "День створення штату"),
            ("2023-01-23", "Євангельський день"),
            ("2023-01-25", "Річниця міста Сан-Паулу"),
            ("2023-02-20", "Карнавал"),
            ("2023-02-21", "Карнавал"),
            ("2023-02-22", "Попільна середа"),
            ("2023-03-06", "День Пернамбуканської революції 1817 року"),
            ("2023-03-10", "Міжнародний жіночий день"),
            ("2023-03-18", "День автономії"),
            ("2023-03-19", "День Святого Йосипа"),
            ("2023-03-25", "День скасування рабства в Сеарі"),
            ("2023-04-07", "Страсна пʼятниця"),
            ("2023-04-17", "День Богоматері Пенья"),
            ("2023-04-21", "День Тирадентіса; День заснування Бразиліа; День страти Тирадентіса"),
            ("2023-04-23", "День Святого Георгія"),
            ("2023-05-01", "День трудящих"),
            ("2023-06-08", "Свято Тіла і Крові Христових"),
            ("2023-06-15", "День заснування Акрі"),
            ("2023-06-18", "Євангельський день"),
            ("2023-06-24", "День Святого Івана"),
            ("2023-06-29", "День Святого Петра"),
            ("2023-07-02", "День незалежності Баїї"),
            ("2023-07-08", "День політичного звільнення Сержипі"),
            ("2023-07-09", "День Конституціоналістської революції"),
            ("2023-07-25", "День Святого Якова"),
            ("2023-07-26", "День заснування міста Гояс"),
            ("2023-07-28", "День приєдання Мараньяна до незалежності Бразилії"),
            ("2023-08-05", "День заснування штату"),
            ("2023-08-07", "День Ріо-Гранді-ду-Норті"),
            ("2023-08-13", "День штату Санта-Катарина"),
            (
                "2023-08-15",
                "День Богоматері Внебовзяття; День приєдання Гран-Пара до незалежності Бразилії",
            ),
            ("2023-09-05", "День піднесення Амазонас до категорії провінцій"),
            ("2023-09-07", "День незалежності Бразилії"),
            ("2023-09-08", "День Амазонії; День Богоматері Різдва"),
            ("2023-09-13", "День створення федеральної території"),
            ("2023-09-16", "День політичного звільнення Алагоаса"),
            ("2023-09-20", "День Гаучо"),
            ("2023-10-03", "День мучеників Куньяу та Уруасу"),
            ("2023-10-05", "День створення штату"),
            ("2023-10-11", "День створення штату"),
            ("2023-10-12", "День Богоматері Апаресіди"),
            ("2023-10-19", "День Піауї"),
            ("2023-10-24", "День заснування Гоянії"),
            ("2023-10-28", "День громадського службовця"),
            ("2023-11-02", "День усіх померлих"),
            ("2023-11-15", "День Богоматері Росіо; День проголошення республіки"),
            ("2023-11-17", "День підписання Петрополіського договору"),
            ("2023-11-20", "День свідомості темношкірих"),
            ("2023-11-26", "День Святої Катерини Александрійської"),
            ("2023-11-30", "Євангельський день"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-31", "Переддень Нового року"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import importlib
import inspect
import warnings
from unittest import TestCase

import pytest

import holidays
from holidays import countries, financial, registry
from tests.common import PYTHON_LATEST_SUPPORTED_VERSION, PYTHON_VERSION


class TestEntityLoader(TestCase):
    @pytest.mark.skipif(
        PYTHON_VERSION != PYTHON_LATEST_SUPPORTED_VERSION,
        reason="Run once on the latest Python version only",
    )
    def test_countries_imports(self):
        warnings.simplefilter("ignore")

        loader_entities = set()
        for module, entities in registry.COUNTRIES.items():
            module = importlib.import_module(f"holidays.countries.{module}")
            for entity in entities:
                countries_cls = getattr(countries, entity)
                loader_cls = getattr(holidays, entity)
                module_cls = getattr(module, entity)

                self.assertIsNotNone(countries_cls, entity)
                self.assertIsNotNone(loader_cls, entity)
                self.assertIsNotNone(module_cls, entity)
                self.assertEqual(countries_cls, module_cls)
                self.assertIsInstance(loader_cls, registry.EntityLoader)
                self.assertIsInstance(loader_cls(), countries_cls)
                self.assertIsInstance(loader_cls(), module_cls)

                loader_entities.add(loader_cls.__name__)

        countries_entities = {
            entity[0] for entity in inspect.getmembers(countries, inspect.isclass)
        }
        self.assertEqual(
            countries_entities,
            loader_entities,
            "Registry entities and countries entities don't match: "
            f"{countries_entities.difference(loader_entities)}",
        )

    def test_country_str(self):
        self.assertEqual(
            str(registry.EntityLoader("holidays.countries.united_states.US")),
            "A lazy loader for <class 'holidays.countries.united_states.US'>. "
            "For inheritance please use the "
            "'holidays.countries.united_states.US' class directly.",
        )

    @pytest.mark.skipif(
        PYTHON_VERSION != PYTHON_LATEST_SUPPORTED_VERSION,
        reason="Run once on the latest Python version only",
    )
    def test_financial_imports(self):
        loader_entities = set()
        for module, entities in registry.FINANCIAL.items():
            module = importlib.import_module(f"holidays.financial.{module}")
            for entity in entities:
                financial_cls = getattr(financial, entity)
                loader_cls = getattr(holidays, entity)
                module_cls = getattr(module, entity)

                self.assertIsNotNone(financial_cls, entity)
                self.assertIsNotNone(loader_cls, entity)
                self.assertIsNotNone(module_cls, entity)
                self.assertEqual(financial_cls, module_cls)
                self.assertIsInstance(loader_cls, registry.EntityLoader)
                self.assertIsInstance(loader_cls(), financial_cls)
                self.assertIsInstance(loader_cls(), module_cls)

                loader_entities.add(loader_cls.__name__)

        financial_entities = {
            entity[0] for entity in inspect.getmembers(financial, inspect.isclass)
        }
        self.assertEqual(
            financial_entities,
            loader_entities,
            "Registry entities and financial entities don't match: "
            f"{financial_entities.difference(loader_entities)}",
        )

    def test_financial_str(self):
        self.assertEqual(
            str(registry.EntityLoader("holidays.financial.ny_stock_exchange.NYSE")),
            "A lazy loader for "
            "<class 'holidays.financial.ny_stock_exchange.NYSE'>. "
            "For inheritance please use the "
            "'holidays.financial.ny_stock_exchange.NYSE' class directly.",
        )

    def test_get_country_codes(self):
        country_codes = set(registry.EntityLoader.get_country_codes(include_aliases=False))
        for entity_classes in registry.COUNTRIES.values():
            self.assertNotIn(entity_classes[0], country_codes)
            self.assertIn(entity_classes[1], country_codes)
            for code in entity_classes[2:]:
                self.assertNotIn(code, country_codes)

    def test_get_country_codes_aliases(self):
        country_codes = set(registry.EntityLoader.get_country_codes(include_aliases=True))
        for entity_classes in registry.COUNTRIES.values():
            self.assertNotIn(entity_classes[0], country_codes)
            for code in entity_classes[1:]:
                if code.isupper():
                    self.assertIn(code, country_codes)
                else:
                    self.assertNotIn(code, country_codes)

    def test_get_financial_codes(self):
        financial_codes = set(registry.EntityLoader.get_financial_codes(include_aliases=False))
        for entity_classes in registry.FINANCIAL.values():
            self.assertNotIn(entity_classes[0], financial_codes)
            self.assertIn(entity_classes[1], financial_codes)
            for code in entity_classes[2:]:
                self.assertNotIn(code, financial_codes)

    def test_get_financial_codes_aliases(self):
        financial_codes = set(registry.EntityLoader.get_financial_codes(include_aliases=True))
        for entity_classes in registry.FINANCIAL.values():
            self.assertNotIn(entity_classes[0], financial_codes)
            for code in entity_classes[1:]:
                if code.isupper():
                    self.assertIn(code, financial_codes)
                else:
                    self.assertNotIn(code, financial_codes)

    def test_inheritance(self):
        def create_instance(parent):
            class SubClass(parent):
                pass

            return SubClass()

        for cls in (holidays.UnitedStates, holidays.US, holidays.USA):
            self.assertIsInstance(cls, holidays.registry.EntityLoader)
            with self.assertRaises(TypeError):
                create_instance(cls)

        for cls in (
            holidays.countries.UnitedStates,
            holidays.countries.US,
            holidays.countries.USA,
        ):
            self.assertIsInstance(create_instance(cls), holidays.countries.UnitedStates)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.saint_lucia import SaintLucia
from tests.common import CommonCountryTests


class TestSaintLucia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(
            SaintLucia, years=range(1979, 2050), years_non_observed=range(1979, 2050)
        )

    def test_new_years(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1979, 2050)))
        self.assertHolidayName(
            "New Year's Holiday", (f"{year}-01-02" for year in range(1979, 2050))
        )

    def test_independence_day(self):
        name = "Independence Day"
        self.assertHolidayName(name, (f"{year}-02-22" for year in range(1979, 2050)))
        dt = (
            "2004-02-23",
            "2009-02-23",
            "2015-02-23",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_good_friday(self):
        name = "Good Friday"
        dts = (
            "1999-04-02",
            "2000-04-21",
            "2010-04-02",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1979, 2050))

    def test_easter_monday(self):
        name = "Easter Monday"
        dts = (
            "1999-04-05",
            "2000-04-24",
            "2010-04-05",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
        )
        self.assertHolidayName(name, dts)
        self.assertHolidayName(name, range(1979, 2050))

    def test_emancipation_day(self):
        name = "Emancipation Day"
        self.assertHolidayName(name, (f"{year}-08-01" for year in range(1979, 2050)))
        dt = (
            "2004-08-02",
            "2010-08-02",
            "2021-08-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_labour_day(self):
        name = "Labour Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1979, 2050)))
        dt = (
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_corpus_christi(self):
        name = "Corpus Christi"
        dt = (
            "2018-05-31",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
            "2024-05-30",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1979, 2050))

    def test_thanksgiving_day(self):
        name = "Thanksgiving Day"
        dt = (
            "1995-10-02",
            "1996-10-07",
            "1997-10-06",
            "1998-10-05",
            "1999-10-04",
            "2000-10-02",
            "2001-10-01",
            "2002-10-07",
            "2003-10-06",
            "2004-10-04",
            "2005-10-03",
            "2006-10-02",
            "2007-10-01",
            "2008-10-06",
            "2009-10-05",
            "2010-10-04",
            "2011-10-03",
            "2012-10-01",
            "2013-10-07",
            "2014-10-06",
            "2015-10-05",
            "2016-10-03",
            "2017-10-02",
            "2018-10-01",
            "2019-10-07",
            "2020-10-05",
            "2021-10-04",
            "2022-10-03",
            "2023-10-02",
            "2024-10-07",
            "2025-10-06",
        )
        self.assertHolidayName(name, dt)
        self.assertHolidayName(name, range(1979, 2050))

    def test_national_day(self):
        name = "National Day"
        self.assertHolidayName(name, (f"{year}-12-13" for year in range(1979, 2050)))
        dt = (
            "2009-12-14",
            "2015-12-14",
            "2020-12-14",
        )
        self.assertHolidayName(f"{name} (observed)", dt)
        self.assertNoNonObservedHoliday(dt)

    def test_christmas_day(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1979, 2050)))

    def test_boxing_day(self):
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in range(1979, 2050)))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Holiday"),
            ("2023-01-03", "New Year's Day (observed)"),
            ("2023-02-22", "Independence Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Labour Day"),
            ("2023-05-29", "Whit Monday"),
            ("2023-06-08", "Corpus Christi"),
            ("2023-08-01", "Emancipation Day"),
            ("2023-10-02", "Thanksgiving Day"),
            ("2023-12-13", "National Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-02", "New Year's Holiday"),
            ("2022-01-03", "New Year's Holiday (observed)"),
            ("2022-02-22", "Independence Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Labor Day (observed)"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-01", "Emancipation Day"),
            ("2022-10-03", "Thanksgiving Day"),
            ("2022-12-13", "National Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.costa_rica import CostaRica
from tests.common import CommonCountryTests


class TestCostaRica(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        cls.full_range = range(1980, 2050)
        super().setUpClass(CostaRica)

    def test_new_year_day(self):
        self.assertHolidayName("Año Nuevo", (f"{year}-01-01" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Jueves Santo"
        self.assertHolidayName(
            name,
            "2016-03-24",
            "2017-04-13",
            "2018-03-29",
            "2019-04-18",
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
        )
        self.assertHolidayName(name, self.full_range)

    def test_good_friday(self):
        name = "Viernes Santo"
        self.assertHolidayName(
            name,
            "2016-03-25",
            "2017-04-14",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
        )
        self.assertHolidayName(name, self.full_range)

    def test_juan_santamaria_day(self):
        name = "Día de Juan Santamaría"
        years_observed = {2006, 2007, 2008, 2023, 2024}
        self.assertHolidayName(
            name, (f"{year}-04-11" for year in self.full_range if year not in years_observed)
        )

        obs_dts = (
            "2006-04-17",
            "2007-04-16",
            "2008-04-14",
            "2023-04-10",
            "2024-04-15",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(name, (f"{year}-04-11" for year in years_observed))

    def test_labor_day(self):
        name = "Día Internacional del Trabajo"
        years_observed = {2021}
        self.assertHolidayName(
            name, (f"{year}-05-01" for year in self.full_range if year not in years_observed)
        )

        obs_dts = "2021-05-03"
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(name, "2021-05-01")

    def test_annexation(self):
        name = "Anexión del Partido de Nicoya a Costa Rica"
        years_observed = {2006, 2007, 2008, 2020, 2021, 2023, 2024}
        self.assertHolidayName(
            name, (f"{year}-07-25" for year in self.full_range if year not in years_observed)
        )

        obs_dts = (
            "2006-07-31",
            "2007-07-30",
            "2008-07-28",
            "2020-07-27",
            "2021-07-26",
            "2023-07-24",
            "2024-07-29",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(name, (f"{year}-07-25" for year in years_observed))

    def test_feast_our_lady_of_angels(self):
        name = "Fiesta de Nuestra Señora de los Ángeles"
        self.assertOptionalHolidayName(name, (f"{year}-08-02" for year in self.full_range))
        self.assertNoHolidayName(name)

    def test_mothers_day(self):
        name = "Día de la Madre"
        years_observed = {2006, 2007, 2020, 2023, 2024}
        self.assertHolidayName(
            name, (f"{year}-08-15" for year in self.full_range if year not in years_observed)
        )

        obs_dts = (
            "2006-08-21",
            "2007-08-20",
            "2020-08-17",
            "2023-08-14",
            "2024-08-19",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(name, (f"{year}-08-15" for year in years_observed))

    def test_black_person_day(self):
        name = "Día de la Persona Negra y la Cultura Afrocostarricense"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name, (f"{year}-08-31" for year in range(2024, self.end_year))
        )
        self.assertNoOptionalHoliday(f"{year}-08-31" for year in range(self.start_year, 2021))
        self.assertNoOptionalHolidayName(name, range(self.start_year, 2021))

        obs_dts = (
            "2021-09-05",
            "2022-09-04",
            "2023-09-03",
        )
        self.assertOptionalHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoOptionalNonObservedHoliday(obs_dts)
        self.assertOptionalNonObservedHolidayName(
            name,
            "2021-08-31",
            "2022-08-31",
            "2023-08-31",
        )

    def test_independence_day(self):
        name = "Día de la Independencia"
        years_observed = {2020, 2021, 2022}
        self.assertHolidayName(
            name, (f"{year}-09-15" for year in self.full_range if year not in years_observed)
        )

        obs_dts = (
            "2020-09-14",
            "2021-09-13",
            "2022-09-19",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(name, (f"{year}-09-15" for year in years_observed))

    def test_cultures_day(self):
        name = "Día de las Culturas"
        self.assertHolidayName(
            name,
            "2008-10-12",
            "2009-10-12",
            "2013-10-12",
            "2014-10-12",
            "2015-10-12",
            "2019-10-12",
        )
        self.assertNoHolidayName(name, range(2020, self.end_year))

        obs_dts = (
            "2010-10-18",
            "2011-10-17",
            "2012-10-15",
            "2016-10-17",
            "2017-10-16",
            "2018-10-15",
        )
        self.assertHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)
        self.assertNonObservedHolidayName(
            name,
            "2010-10-12",
            "2011-10-12",
            "2012-10-12",
            "2016-10-12",
            "2017-10-12",
            "2018-10-12",
        )

    def test_army_abolition_day(self):
        name = "Día de la Abolición del Ejército"
        self.assertNoHolidayName(name)
        self.assertOptionalHolidayName(
            name, (f"{year}-12-01" for year in range(2023, self.end_year))
        )
        self.assertNoOptionalHoliday("2019-12-01")
        self.assertNoOptionalHolidayName(name, range(self.start_year, 2020))

        obs_dts = (
            "2020-11-30",
            "2021-11-29",
            "2022-12-05",
        )
        self.assertOptionalHolidayName(f"{name} (observado)", obs_dts)
        self.assertNoOptionalNonObservedHoliday(obs_dts)
        self.assertOptionalNonObservedHolidayName(
            name,
            "2020-12-01",
            "2021-12-01",
            "2022-12-01",
        )

    def test_christmas_day(self):
        self.assertHolidayName("Navidad", (f"{year}-12-25" for year in self.full_range))

    def test_2022(self):
        self.assertHolidaysInYear(
            2022,
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-11", "Día de Juan Santamaría"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día Internacional del Trabajo"),
            ("2022-07-25", "Anexión del Partido de Nicoya a Costa Rica"),
            ("2022-08-15", "Día de la Madre"),
            ("2022-09-19", "Día de la Independencia (observado)"),
            ("2022-12-25", "Navidad"),
        )

        self.assertNonObservedHolidaysInYear(
            2022,
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-11", "Día de Juan Santamaría"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día Internacional del Trabajo"),
            ("2022-07-25", "Anexión del Partido de Nicoya a Costa Rica"),
            ("2022-08-15", "Día de la Madre"),
            ("2022-09-15", "Día de la Independencia"),
            ("2022-12-25", "Navidad"),
        )

    def test_optional_2022(self):
        self.assertOptionalHolidaysInYear(
            2022,
            ("2022-08-02", "Fiesta de Nuestra Señora de los Ángeles"),
            ("2022-09-04", "Día de la Persona Negra y la Cultura Afrocostarricense (observado)"),
            ("2022-12-05", "Día de la Abolición del Ejército (observado)"),
        )

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Año Nuevo"),
            ("2023-04-06", "Jueves Santo"),
            ("2023-04-07", "Viernes Santo"),
            ("2023-04-10", "Día de Juan Santamaría (observado)"),
            ("2023-05-01", "Día Internacional del Trabajo"),
            ("2023-07-24", "Anexión del Partido de Nicoya a Costa Rica (observado)"),
            ("2023-08-14", "Día de la Madre (observado)"),
            ("2023-09-15", "Día de la Independencia"),
            ("2023-12-25", "Navidad"),
        )

    def test_optional_2023(self):
        self.assertOptionalHolidaysInYear(
            2023,
            ("2023-08-02", "Fiesta de Nuestra Señora de los Ángeles"),
            ("2023-09-03", "Día de la Persona Negra y la Cultura Afrocostarricense (observado)"),
            ("2023-12-01", "Día de la Abolición del Ejército"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Año Nuevo"),
            ("2022-04-11", "Día de Juan Santamaría"),
            ("2022-04-14", "Jueves Santo"),
            ("2022-04-15", "Viernes Santo"),
            ("2022-05-01", "Día Internacional del Trabajo"),
            ("2022-07-25", "Anexión del Partido de Nicoya a Costa Rica"),
            ("2022-08-02", "Fiesta de Nuestra Señora de los Ángeles"),
            ("2022-08-15", "Día de la Madre"),
            ("2022-09-04", "Día de la Persona Negra y la Cultura Afrocostarricense (observado)"),
            ("2022-09-19", "Día de la Independencia (observado)"),
            ("2022-12-05", "Día de la Abolición del Ejército (observado)"),
            ("2022-12-25", "Navidad"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-11", "Juan Santamaría Day"),
            ("2022-04-14", "Maundy Thursday"),
            ("2022-04-15", "Good Friday"),
            ("2022-05-01", "International Labor Day"),
            ("2022-07-25", "Annexation of the Party of Nicoya to Costa Rica"),
            ("2022-08-02", "Feast of Our Lady of the Angels"),
            ("2022-08-15", "Mother's Day"),
            ("2022-09-04", "Day of the Black Person and Afro-Costa Rican Culture (observed)"),
            ("2022-09-19", "Independence Day (observed)"),
            ("2022-12-05", "Army Abolition Day (observed)"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-11", "День Хуана Сантамарії"),
            ("2022-04-14", "Великий четвер"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-05-01", "Міжнародний день праці"),
            ("2022-07-25", "День приєднання Нікої"),
            ("2022-08-02", "Свято Богоматері Ангелів"),
            ("2022-08-15", "День матері"),
            ("2022-09-04", "День чорношкірої людини та афро-костариканської культури (вихідний)"),
            ("2022-09-19", "День незалежності (вихідний)"),
            ("2022-12-05", "День ліквідації армії (вихідний)"),
            ("2022-12-25", "Різдво Христове"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.calendars.gregorian import (
    TUE,
    SAT,
    _timedelta,
    _get_nth_week_of_month,
    _get_nth_weekday_of_month,
)


class TestGregorianCalendar(TestCase):
    def test_get_nth_week_of_month(self):
        # Week 1...4.
        self.assertEqual(_get_nth_week_of_month(1, 1, 2024), date(2024, 1, 1))
        self.assertEqual(_get_nth_week_of_month(2, 1, 2024), date(2024, 1, 8))
        self.assertEqual(_get_nth_week_of_month(3, 1, 2024), date(2024, 1, 15))
        self.assertEqual(_get_nth_week_of_month(4, 1, 2024), date(2024, 1, 22))
        # No Week 5 in Jan 2024.
        with self.assertRaises(ValueError):
            _get_nth_week_of_month(5, 1, 2024)

        # Week 1...5.
        self.assertEqual(_get_nth_week_of_month(1, 9, 2024), date(2024, 8, 26))
        self.assertEqual(_get_nth_week_of_month(2, 9, 2024), date(2024, 9, 2))
        self.assertEqual(_get_nth_week_of_month(3, 9, 2024), date(2024, 9, 9))
        self.assertEqual(_get_nth_week_of_month(4, 9, 2024), date(2024, 9, 16))
        self.assertEqual(_get_nth_week_of_month(5, 9, 2024), date(2024, 9, 23))

        # Week -1...-4.
        self.assertEqual(_get_nth_week_of_month(-1, 8, 2024), date(2024, 8, 19))
        self.assertEqual(_get_nth_week_of_month(-2, 8, 2024), date(2024, 8, 12))
        self.assertEqual(_get_nth_week_of_month(-3, 8, 2024), date(2024, 8, 5))
        self.assertEqual(_get_nth_week_of_month(-4, 8, 2024), date(2024, 7, 29))
        # No Week -5 in Aug 2024.
        with self.assertRaises(ValueError):
            _get_nth_week_of_month(-5, 8, 2024)

        # Week -1...-5.
        self.assertEqual(_get_nth_week_of_month(-1, 12, 2024), date(2024, 12, 23))
        self.assertEqual(_get_nth_week_of_month(-2, 12, 2024), date(2024, 12, 16))
        self.assertEqual(_get_nth_week_of_month(-3, 12, 2024), date(2024, 12, 9))
        self.assertEqual(_get_nth_week_of_month(-4, 12, 2024), date(2024, 12, 2))
        self.assertEqual(_get_nth_week_of_month(-5, 12, 2024), date(2024, 11, 25))

        # Week ordinals start at 1 (or negatives from the end); zero is invalid.
        with self.assertRaises(ValueError):
            _get_nth_week_of_month(0, 1, 2024)

        # Oversized n for December.
        with self.assertRaises(ValueError):
            _get_nth_week_of_month(53, 12, 2023)

    def test_get_nth_weekday_of_month(self):
        year = 2023
        # The first Tuesdays of 2023.
        for month, day in enumerate((3, 7, 7, 4, 2, 6, 4, 1, 5, 3, 7, 5), 1):
            first_tuesday = _get_nth_weekday_of_month(1, TUE, month, year)
            self.assertEqual(first_tuesday.day, day)

        # The last Saturdays of 2023.
        for month, day in enumerate((28, 25, 25, 29, 27, 24, 29, 26, 30, 28, 25, 30), 1):
            last_friday = _get_nth_weekday_of_month(-1, SAT, month, year)
            self.assertEqual(last_friday.day, day)

    def test_timedelta(self):
        for ymd1, ymd2 in (
            ((2023, 1, 1), (2023, 1, 6)),
            ((2023, 1, 31), (2023, 2, 5)),
            ((2023, 2, 24), (2023, 3, 1)),
            ((2023, 2, 26), (2023, 3, 3)),
            ((2024, 1, 1), (2024, 1, 6)),
            ((2024, 1, 31), (2024, 2, 5)),
            ((2024, 2, 24), (2024, 2, 29)),
            ((2024, 2, 26), (2024, 3, 2)),
        ):
            dt1 = date(*ymd1)
            dt2 = date(*ymd2)
            self.assertEqual(_timedelta(dt1, +5), dt2)
            self.assertEqual(_timedelta(dt2, -5), dt1)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.groups import PersianCalendarHolidays
from holidays.holiday_base import HolidayBase


class TestPersianCalendarHolidays(TestCase):
    def test_add_persian_calendar_holiday(self):
        # Check for out-of-range dates.
        class TestHolidays(HolidayBase, PersianCalendarHolidays):
            end_year = 2102

            def __init__(self, *args, **kwargs):
                PersianCalendarHolidays.__init__(self)
                super().__init__(*args, **kwargs)

        test_holidays = TestHolidays()

        test_holidays._populate(2102)
        test_holidays._add_nowruz_day("Persian New Year")
        test_holidays._add_islamic_republic_day("Islamic Republic Day")
        test_holidays._add_natures_day("Nature's Day")
        test_holidays._add_death_of_khomeini_day("Death of Khomeini")
        test_holidays._add_khordad_uprising_day("Khordad National Uprising")
        test_holidays._add_islamic_revolution_day("Islamic Revolution Day")
        test_holidays._add_oil_nationalization_day("Iranian Oil Industry Nationalization Day")
        self.assertEqual(0, len(test_holidays))
